/*
 * Created on Jan 28, 2005
 *
 */
package br.unicamp.nied.teleduc.extrator.ferramentas;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URI;

/**
 * @author leonel
 * 
 * <p>
 * Representa um arquivo anexo a um item de uma ferramenta.
 * </p>
 * <p>
 * Arquivos anexos a um item contm um marcador booleano "especial". Em algumas
 * ferramentas, por exemplo, Material de apoio e similares, este marcador indica
 * que o arquivo anexo  oculto e no deve ser exibido ao exibir outras
 * informaes sobre o item. Em outras, como agenda, indica que este  o nico
 * arquivo anexo que deve ser exibido. O significado do marcador e seu uso ficam
 * a cargo da ferramenta.
 * </p>
 *  
 */
public class Anexo extends File {
    private boolean isEspecial;
    private String  caminhoExtracao;
    private String  caminhoSemPrefixo;

    /**
     * @param pathname
     * @throws IOException
     */
    public Anexo(String pathname) {
        super(pathname);
        init();
    }

    /**
     * @param uri
     * @throws IOException
     */
    public Anexo(URI uri) {
        super(uri);
        init();
    }

    /**
     * @param parent
     * @param child
     * @throws IOException
     */
    public Anexo(File parent, String child) {
        super(parent, child);
        init();
    }

    /**
     * @param parent
     * @param child
     * @throws IOException
     */
    public Anexo(String parent, String child) {
        super(parent, child);
        init();
    }
    
    /**
     * Mtodo de inicializao do objeto, chamado pelos construtores.
     * 
     * @return
     * @throws IOException
     */
    private void init() {
        this.caminhoSemPrefixo = this.getAbsolutePath();
        if (this.exists() && (!this.isDirectory())) {
            this.setEspecial(this.verificaStatus());
        } else {
            this.setEspecial(false);
        }
    }
    
    /**
     * Verifica se o arquivo em disco representado por este objeto tem o status
     * especial
     * 
     * @return true se o arquivo em disco tem o status especial, false caso
     *         contrrio
     * @throws IOException
     */
    public boolean verificaStatus() {
        if (this.isDirectory()) {
            return false;
        }
        try {
            // Executar o comando
            String command = "stat --format=%a " + this.getAbsolutePath();
            Process child = Runtime.getRuntime().exec(command);

            // Pegar o retorno do comando e interpretar
            BufferedReader in = new BufferedReader(new InputStreamReader(child
                    .getInputStream()));
            String strPerms = in.readLine();

            // Ler a permisso em um octal
            int perms = Integer.parseInt(strPerms, 8);
            /*
             * O TelEduc usa o bit de permisso de escrita do grupo do arquivo
             * como flag: se o bit estiver ligado, a flag  true, seno false.
             * 
             * Para recuperar este bit, fao uma mscara que zera todos os bits
             * exceto o bit de escrita do arquivo. Lembre que os bits de
             * permisso so:
             * rwxr(w)xrwx
             *      ^---- queremos este bit
             * Se o bit estiver ligado, a operao de mascara vai retornar maior
             * que zero. Seno, a mscara retorna zero.
             */
            if ((perms & 0020) == 0) {
                // O bit est desligado, retorne false
                return false;
            } else {
                // O bit est ligado
                return true;
            }
        } catch (NumberFormatException ex) {
            return false;
        } catch (IOException ioex) {
            return false;
        }
    }

    /**
     * Indica se este arquivo anexo  marcado como especial.
     * 
     * @return true se o arquivo anexo for marcado como especial, false caso
     *         contrrio
     */
    public boolean isEspecial() {
        return isEspecial;
    }

    /**
     * Marca ou desmarca a propriedade "especial" deste anexo.
     * 
     * @param isEspecial
     *            booleano para ligar ou desligar a propriedade "especial"
     */
    public void setEspecial(boolean isEspecial) {
        this.isEspecial = isEspecial;
    }

    /**
     * <p>
     * Retorna uma String contendo o nome do primeiro diretorio abaixo da raiz
     * da extrao.
     * </p>
     * <p>
     * Ao extrair um curso, os arquivos anexos so copiados para um diretorio no
     * formato:
     * {dirDestino}/{nomeCurso}/{nomeUsuario}/arquivos/{ferramenta}/{idItem}/anexo
     * Este mtodo extrai o prefixo at o anexo e retorna a rvore de diretrios
     * a partir do anexo.
     * </p>
     * <p>
     * Se o arquivo estiver ainda mais fundo na rvore de diretrios, como:
     * {dirDestino}/{nomeCurso}/{nomeUsuario}/arquivos/{ferramenta}/{idItem}/animal/mamifero/cao.txt
     * o mtodo retorna "animal/mamifero/cao.txt"
     * </p>
     * <p>
     * Para usar, vc deve primeiro executar setCaminhoExtracao(String) para
     * setar o caminho prefixo onde ser extraido o curso.
     * 
     * @return String representando um diretrio na rvore do anexo.
     */
    public String getCaminhoSemPrefixoExtracao() {
        return this.caminhoSemPrefixo;
    }

    /**
     * @return Returns the caminhoExtracao.
     */
    public String getCaminhoExtracao() {
        return caminhoExtracao;
    }

    /**
     * @param caminhoExtracao
     *            The caminhoExtracao to set.
     */
    public void setCaminhoExtracao(String caminhoExtracao) {
        this.caminhoExtracao = caminhoExtracao;
        /*
         * setar o caminho sem prefixo baseado no prefixo e no caminho deste
         * arquivo.
         */
        if (this.getAbsolutePath().indexOf(caminhoExtracao) > -1) {
            this.caminhoSemPrefixo = this.getAbsolutePath().substring(
                    caminhoExtracao.length());
        }
    }
}