/*
 * Created on Feb 22, 2005
 *
 */
package br.unicamp.nied.teleduc.extrator.ferramentas.acessos;

import java.io.IOException;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import br.unicamp.nied.teleduc.extrator.curso.Curso;
import br.unicamp.nied.teleduc.extrator.curso.ExtractionException;
import br.unicamp.nied.teleduc.extrator.database.ConnectionFactory;
import br.unicamp.nied.teleduc.extrator.database.DBException;
import br.unicamp.nied.teleduc.extrator.usuario.Usuario;
import br.unicamp.nied.teleduc.extrator.usuario.UsuarioFactory;

/**
 * @author leonel
 *
 * Representa o conjunto de acessos dos usurios a um curso ou a determinada
 * ferramenta do curso
 *
 */
public class AcessosCurso {
    private Curso curso;

    private UsuarioFactory usuarioFactory;

    private Map ultimosAcessos;

    private Map quantidadesAcessos;

    private Map quantidadesAcessosNormalizadas;

    private Date primeiroAcesso;

    private Date ultimoAcesso;

    /**
     * @throws ExtractionException
     *
     */
    public AcessosCurso(Curso curso) throws ExtractionException {
        super();
        this.curso = curso;
        clear();
        init();
    }

    public void clear() throws ExtractionException {
        this.usuarioFactory = UsuarioFactory.getInstance(this.curso);
        this.ultimosAcessos = new HashMap();
        this.quantidadesAcessos = new HashMap();
        this.quantidadesAcessosNormalizadas = new HashMap();
    }

    public void refresh() throws ExtractionException {
        init();
    }

    /**
     * Indica a quantidade de acessos deste usurio ao curso
     *
     * @param usuario
     *            Usurio do qual queremos saber a quantidade de acessos.
     * @return
     */
    public int quantidadeAcessos(Usuario usuario) {
        Integer qtdeAcessos = (Integer) this.quantidadesAcessos.get(usuario);
        if (qtdeAcessos == null) {
            return 0;
        } else {
            return qtdeAcessos.intValue();
        }
    }

    /**
     * Retorna a quantidade de acessos deste usurio normalizada em relao aos
     * outros usurios do curso. O usurio com mais acessos tem uma quantidade
     * normalizada igual a 1.0, os demais tem uma quantidade normalizada
     * proporcional
     *
     */
    public float quantidadeAcessosNormalizada(Usuario usuario) {
        Float norm = (Float) this.quantidadesAcessosNormalizadas.get(usuario);
        float ret = 0.0F;
        if (norm != null) {
            ret = norm.floatValue();
        }
        return ret;
    }

    /**
     * Data de ltimo acesso do usurio ao curso.
     *
     * @param usuario
     *            Usurio do qual queremos saber a data de ltimo acesso
     * @return
     */
    public Date ultimoAcesso(Usuario usuario) {
        Date ultimoAcesso = (Date) this.ultimosAcessos.get(usuario);
        return ultimoAcesso;
    }

    public Date getUltimoAcesso() {
        return this.ultimoAcesso;
    }

    public Date getPrimeiroAcesso() {
        return this.primeiroAcesso;
    }

    /**
     * Inicializa o objeto pesquisando na base de dados quais os ltimos acessos
     * dos usurios, a quantidade de acessos de cada um e quando ocorreram o
     * primeiro e o ltimo acesso ao curso.
     *
     * @throws ExtractionException
     */
    private void init() throws ExtractionException {
        initPrimeiroUltimoAcessos();
        initAcessos();
    }

    /**
     * Busca na base de dados quando ocorreram o primeiro e o ltimo acesso ao
     * curso
     */
    private void initPrimeiroUltimoAcessos() throws ExtractionException {
        try {
            String sql = "select min(data) as 'min', max(data) as 'max'"
            + " from Curso_acessos";
            //String sql = "select curso_inicio as 'min', curso_fim as 'max'"
            //        + " from Cursos";
            Connection conn = ConnectionFactory.getConnectionFactory()
                    .getConnection(this.curso.getBaseCurso());
            PreparedStatement stmt = conn.prepareStatement(sql);
            ResultSet rs = stmt.executeQuery();
            if (!rs.next()) {
                throw new ExtractionException("Erro ao buscar primeiro e"
                        + " ltimo acesso do curso");
            }
            this.primeiroAcesso = new Date(rs.getLong("min") * 1000L);
            this.ultimoAcesso = new Date(rs.getLong("max") * 1000L);
        } catch (DBException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } catch (SQLException e) {
            e.printStackTrace();
        }
    }

    /**
     * Busca na base de dados quais os ltimos acessos e a quantidade de acessos
     * de cada usurio no curso
     *
     * @throws ExtractionException
     */
    private void initAcessos() throws ExtractionException {
        try {
            String sql = "select cod_usuario, count(*) 'num_acessos',"
                    + " max(data) 'ultimo_acesso'"
                    + " from Curso_acessos where cod_ferramenta is null"
                    + " and cod_usuario >= 0" + " group by cod_usuario";
            Connection conn = ConnectionFactory.getConnectionFactory()
                    .getConnection(curso.getBaseCurso());
            PreparedStatement stmt = conn.prepareStatement(sql);
            ResultSet rs = stmt.executeQuery();

            int maxAcessos = 0;
            while (rs.next()) {
                Usuario usr = this.usuarioFactory.getUsuario(rs
                        .getInt("cod_usuario"));

                /*
                 * Usurio com maior quantidade de acessos do curso
                 */
                int nAcessos = rs.getInt("num_acessos");
                if (nAcessos > maxAcessos) {
                    maxAcessos = nAcessos;
                }

                /*
                 * Quantidade de acessos deste usurio
                 */
                Integer qtdeAcessos = new Integer(nAcessos);
                this.quantidadesAcessos.put(usr, qtdeAcessos);

                /*
                 * Data de ltimo acesso deste usurio
                 */
                Date ultimoAcesso = new Date(
                        rs.getLong("ultimo_acesso") * 1000L);
                this.ultimosAcessos.put(usr, ultimoAcesso);
            }

            /*
             * Calcular qtdes normalizadas de acessos
             */
            float maxAcessosF = (float) maxAcessos;
            for (Iterator itr = this.quantidadesAcessos.keySet().iterator(); itr
                    .hasNext();) {
                Usuario usr = (Usuario) itr.next();
                float norm = ((float) this.quantidadeAcessos(usr))
                        / maxAcessosF;

                this.quantidadesAcessosNormalizadas.put(usr, new Float(norm));
            }

        } catch (DBException e) {
            throw new ExtractionException("Erro ao extrair acessos", e);
        } catch (IOException e) {
            throw new ExtractionException("Erro ao extrair acessos", e);
        } catch (SQLException e) {
            throw new ExtractionException("Erro ao extrair acessos", e);
        }
    }
}